//
// timezone-make-win-links.cpp
//
// Bill Seymour, 2024-03-03
//
// This is the program that creates symbolic links in a Windows file system
// when using Zoneinfo data.  It's an optional preparation step for using the
// timezone class (see https://www.cstdbill.com/civil-time/timezone.html);.
//
// This file is in the public domain.
//

#include "timezone-win-links.hpp" // includes <string>

#include <iostream> // cerr
#include <utility>  // move
#include <cstddef>  // size_t

namespace {

using std::string;
using std::move;
using std::cerr;
using std::size_t;

namespace tw = timezone_win;

string make_path(string&& fn)
{
    string retval(tw::tzdir);
    retval.append(1, '\\');
    retval.append(move(fn));
    tw::fix_sep(retval);
    return retval;
}

bool make_link(const string& input)
{
    //
    // Just ignore anything with "localtime" in it.
    //
    if (input.find("localtime") != string::npos)
    {
        return true;
    }

    //
    // Find the two names in an input line of the form,
    //
    //   L real-name link-name
    //
    static constexpr size_t real_beg = 2;
    size_t real_end = input.find_first_of(" \t", real_beg);

    size_t link_beg = input.find_first_not_of(" \t", real_end);
    if (link_beg == string::npos)
    {
        cerr << "Invalid input format \"" << input << "\"\n";
        return false;
    }
    size_t link_end = input.find_first_of(" \t", link_beg);
    if (link_end == string::npos)
    {
        link_end = input.size();
    }

    //
    // Make full paths.
    //
    string real_fn(
      move(make_path(move(input.substr(real_beg, real_end - real_beg)))));

    string link_fn(
      move(make_path(move(input.substr(link_beg, link_end - link_beg)))));

    //
    // Make the link.
    //
    if (!tw::make_symlink(link_fn, real_fn))
    {
        cerr << "Error making link " << link_fn
             << " pointing to " << real_fn << '\n';
        return false;
    }

    return true;
}

} // anonymous namespace

int main(int argc, char** argv)
{
    if (argc < 2 || argc > 3)
    {
        cerr << "Usage:  timezone-make-win-links "
                "tz-root-directory [localtime-target]\n";
        return EXIT_FAILURE;
    }
    tw::tzdir.assign(argv[1]);

    if (argc == 3)
    {
        if (!tw::make_symlink(make_path(move(string("localtime"))),
                              make_path(move(string(argv[2])))))
        {
            cerr << "Can't make localtime link\n";
            return EXIT_FAILURE;
        }
    }

    return tw::main_loop(make_link);
}

// End of timezone-make-win-links
