//
// timezone-win-links.cpp
//
// Bill Seymour, 2024-03-03
//
// This is the main loop for both of the programs that are
// optional preparation steps for using Bill's timezone class
// (see https://www.cstdbill.com/civil-time/timezone.html)
// along with Zoneinfo's TZif files on Windows.
//
// Also supplied if you have a C++17 compiler and standard library
// is the <filesystem> function to create symbolic links.  (If you
// don't have at least C++17, you'll need to use the native Windows
// function in timezone-make-one-win-link.cpp.)
//
// When compiling with MSVC (otherwise why are you even doing this?),
// use the /std:c++17 and /Zc:__cplusplus options.
//
// This file is in the public domain.
//

#include "timezone-win-links.hpp" // includes <string>

#include <iostream> // cerr
#include <fstream>  // ifstream

#if __cplusplus >= 201703L
  #include <exception>
  #include <filesystem>
#endif

#include <cstdlib> // EXIT_*

namespace timezone_win {

std::string tzdir;

//
// This is the main loop for both of the programs.  It just passes the Link
// lines read from {tzdir}\tzdata.zi to some function that does the deed
// and returns either EXIT_SUCCESS or EXIT_FAILURE.
//
int main_loop(action act)
{
    std::string input_fn(tzdir);
    input_fn.append(1, '\\');
    input_fn.append("tzdata.zi");
    fix_sep(input_fn);

    std::ifstream istr(input_fn);
    if (!istr)
    {
        std::cerr << "Can't open " << input_fn << '\n';
        return EXIT_FAILURE;
    }

    for (std::string input; std::getline(istr, input); )
    {
        if (input.size() < 2 || input[0] != 'L' || input[1] != ' ')
        {
            continue; // not a Link line
        }
        if (!act(input))
        {
            istr.close();
            return EXIT_FAILURE;
        }
    }

    bool ok = !istr.bad() && istr.eof();
    istr.close();
    if (!ok)
    {
        std::cerr << "Error reading " << input_fn << '\n';
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

#if __cplusplus >= 201703L
//
// This is the standard way to create a symlink, but it requires
// at least a C++17 compiler and standard library.
//
// Note that it just returns whether it's successful or not.
// The caller will generate any required error message.
//
bool make_symlink(const std::string& link_path, const std::string& real_path)
{
    try
    {
        namespace fs = std::filesystem;
        fs::create_symlink(fs::path(real_path, fs::path::native_format),
                           fs::path(link_path, fs::path::native_format));
        return true;
    }
    catch (const std::exception& e)
    {
        std::cerr << e.what() << '\n';
        return false;
    }
}
#endif // C++17

} // namespace timezone_win

// End of timezone-win-links.cpp
